

'
' Program 2 (FREQUENCY OR OFFSET SETTING)
'
' Program taken from the "CT-17 Communication Interface-V (CI-V) Level
' Converter Instruction Manual". Program converted to IBM QBASIC by Bill
' Heaton, N7WRI.
'
DEFINT G-Z

DECLARE SUB DoCommand (Cmd AS INTEGER, Arg AS STRING)
DECLARE SUB GetReply ()
DECLARE SUB SetupBCD ()
DECLARE SUB SetupCOM ()
DECLARE SUB ShowMainPrompts ()

DECLARE FUNCTION GetFreq$ (ByteCnt AS INTEGER)
DECLARE FUNCTION GetOff$ (ByteCnt AS INTEGER)
DECLARE FUNCTION STRTOHEX$ (Str AS STRING)

DIM SHARED BCD$(100)

'
' Configuration Information
'

CONST RA = &H3C                 ' Receive Address (IC-737)
CONST TA = &HE0                 ' Transmit Address (Computer)
CONST FREQCNT = 5               ' Number of bytes in frequency (IC-737 uses 5)

CONST PORT$ = "COM1"            ' Serial Port to use
CONST PORTNO = 1                ' Serial Port to use
CONST CONF$ = "1200,N,8,1"      ' Baud rate, Parity, Bits, Stop Bits

CONST SHOWCOM = 1               ' Show Com Packets ( 0=No, 1=Yes)

'
' Initialize
'

SetupBCD
SetupCOM
ShowMainPrompts

'
'  Endless loop until an event handler kills us
'
DO UNTIL TRUE
LOOP
 
'
' Event Handlers
'
Serial: GetReply:                     RETURN
F1: DoCommand &H5, GetFreq$(FREQCNT): RETURN
F2: DoCommand &HD, GetOff$(FREQCNT):  RETURN
F0: CLOSE : SYSTEM

' +---------------------------------------------------------------------+
' |                                                                     |
' |                      ICOM CI-V Packet Layout                        |
' |     +----------+----------+---------+---------+---------+------+    |
' |     | Preamble | Transmit | Receive | Command | Sub     | EOM  |    |
' |     | <FE><FE> | Address  | Address |         | Command | <FD> |    |
' |     +----------+----------+---------+---------+---------+------+    |
' |                                                                     |
' |     A packet consists of two bytes of &HFE, one byte for the        |
' |     transmit address (Controller), One byte receive address         |
' |     (Rig), one byte command, one to five byte subcommand, and       |
' |     finally the tail of one byte of &HFD.                           |
' |                                                                     |
' +---------------------------------------------------------------------+
SUB DoCommand (Cmd AS INTEGER, SubCmd AS STRING)
 
  '
  ' Create the packet and send it out
  '
  Out$ = CHR$(&HFE) + CHR$(&HFE) + CHR$(RA) + CHR$(TA) + CHR$(Cmd) + SubCmd + CHR$(&HFD)
  PRINT #1, Out$;

  '
  ' If we're watching packets, send to the screen in hex
  '
  IF SHOWCOM THEN
    LOCATE 17, 1: PRINT "Sent: "
    LOCATE 17, 7: PRINT STRTOHEX$(Out$);
    LOCATE 18, 1: PRINT "Echo: "
    LOCATE 19, 1: PRINT "Back: "
  END IF
END SUB

DEFDBL F
'
'  Get a Frequency and convert to BCD then return it
'
FUNCTION GetFreq$ (ByteCnt AS INTEGER)
   
  F = 0!
  WHILE F = 0!
    CLS
    LOCATE 12, 30: INPUT "Enter Frequency (MHz):"; F
    F = F + 1E-12: IF F < .1 OR F > 2147.484 THEN F = 0!
  WEND

  F$ = RIGHT$("0000000000" + MID$(STR$(INT(F * 1000000#)), 2), FREQCNT * 2)

  DA$ = ""
 
  FOR K = 1 TO (FREQCNT * 2) STEP 2
    DA$ = BCD$(VAL(MID$(F$, K, 2))) + DA$
  NEXT K

  GetFreq$ = DA$
END FUNCTION

'
'  Get an offset, convert to BCD and return it
'
FUNCTION GetOff$ (ByteCnt AS INTEGER)
  
  F = 0!
  WHILE F = 0!
    CLS
    LOCATE 12, 30: INPUT "Enter Offset (MHz):"; F
    F = F + 1E-12: IF F < .1 OR F >= 99.9999 THEN F = 0!
  WEND

  F$ = RIGHT$("0000000000" + MID$(STR$(INT(F * 10000#)), 2), 6)

  DA$ = ""

  FOR K = 1 TO 6 STEP 2
    DA$ = BCD$(VAL(MID$(F$, K, 2))) + DA$
  NEXT K

  GetOff$ = DA$
END FUNCTION

DEFSNG F
'
' GetReply      - Character has arrived from rig, stuff it away until
'                 have an entire packet and display it.
'
SUB GetReply
  STATIC Hold$

  '
  ' Accumulate the Reply, if its not end of packet get out early
  '
  Hold$ = Hold$ + INPUT$(LOC(PORTNO), PORTNO)
  IF INSTR(Hold$, CHR$(&HFD)) = 0 THEN
    EXIT SUB
  END IF

  '
  ' Echo replys to the screen if we were told to
  '
  IF SHOWCOM THEN
    IF MID$(Hold$, 3, 1) = CHR$(TA) THEN
      LOCATE 19, 7: PRINT STRTOHEX$(Hold$);
    
      SELECT CASE MID$(Hold$, 5, 1)
        CASE CHR$(&HFB)
            LOCATE 25, 1: PRINT "  [OK]     ";
        CASE CHR$(&HFA)
            LOCATE 25, 1: PRINT "  [ERROR]  ";
        CASE ELSE
            LOCATE 25, 1: PRINT "  [Unknown]";
      END SELECT
    ELSE
      LOCATE 18, 7: PRINT STRTOHEX$(Hold$);
    END IF
  END IF

  ' Get ready for next reply
  Hold$ = ""
END SUB

'
'  Set up the BCD string array
'
SUB SetupBCD
  FOR M = 0 TO 9
    FOR N = 0 TO 9
      BCD$(10 * M + N) = CHR$(16 * M + N)
    NEXT N
  NEXT M
END SUB

'
'  Setup the channel to the serial port
'
SUB SetupCOM

  OPEN PORT$ + ":" + CONF$ + ",CD0,CS0,DS0,OP0,RS" FOR RANDOM AS #1
  ON COM(PORTNO) GOSUB Serial
  COM(PORTNO) ON
END SUB

'
' Paint the buttons, Set the text view, and setup the key event assignments
'
SUB ShowMainPrompts

  '
  '  Paint the prompts
  '
  CLS
  M1$ = "ͻ"
  M2$ = "  F1     F2                                                     F10  "
  M3$ = " Freq   Offset                                                  EXIT "
  M4$ = "ͼ"

  LOCATE 21, 1: PRINT M1$;
  LOCATE 22, 1: PRINT M2$;
  LOCATE 23, 1: PRINT M3$;
  LOCATE 24, 1: PRINT M4$;
  VIEW PRINT 1 TO 20
 
  '
  '  Setup function key handlers for each options
  '
  ON KEY(1) GOSUB F1:  KEY(1) ON
  ON KEY(2) GOSUB F2:  KEY(2) ON
  ON KEY(10) GOSUB F0: KEY(10) ON

END SUB

'
' Translate all the characters in a string to hex and
' return the resulting string.
'
FUNCTION STRTOHEX$ (Str AS STRING)
Scn$ = ""
FOR i = 1 TO LEN(Str)
     C$ = HEX$(ASC(MID$(Str, i, 1)))
     Scn$ = Scn$ + C$ + " "
  NEXT i
STRTOHEX$ = Scn$
END FUNCTION

